@@ -27,18 +27,30 @@ G_BEGIN_DECLS
 
 #define CONF_PREFIX "/apps/rhythmbox"
 
-#define CONF_FIRST_TIME CONF_PREFIX "/first_time_flag"
-#define CONF_AUTO_RATE  CONF_PREFIX "/auto_rate"
+#define CONF_FIRST_TIME CONF_PREFIX   "/first_time_flag"
+#define CONF_GRACE_PERIOD CONF_PREFIX "/grace_period"
+#define CONF_LIBRARY_LOCATION	CONF_PREFIX "/library_locations"
+#define CONF_MONITOR_LIBRARY	CONF_PREFIX "/monitor_library"
 #define CONF_UI_DIR               CONF_PREFIX "/ui"
 #define CONF_UI_STATUSBAR_HIDDEN  CONF_PREFIX "/ui/statusbar_hidden"
+#define CONF_UI_TOOLBAR_HIDDEN  CONF_PREFIX "/ui/toolbar_hidden"
+#define CONF_UI_SONG_POSITION_SLIDER_HIDDEN  CONF_PREFIX "/ui/song_position_slider_hidden"
 #define CONF_UI_SOURCELIST_HIDDEN CONF_PREFIX "/ui/sourcelist_hidden"
+#define CONF_UI_QUEUE_AS_SIDEBAR  CONF_PREFIX "/ui/queue_as_sidebar" 
 #define CONF_UI_SMALL_DISPLAY     CONF_PREFIX "/ui/small_display"
+#define CONF_UI_FULLSCREEN     CONF_PREFIX "/ui/fullscreen"
 #define CONF_UI_OBSOLETE_COLUMNS_SETUP CONF_PREFIX "/ui/columns_setup"
 #define CONF_UI_COLUMNS_SETUP CONF_PREFIX "/ui/rhythmdb_columns_setup"
 #define CONF_UI_TIME_DISPLAY CONF_PREFIX "/ui/time_display"
 #define CONF_UI_BROWSER_VIEWS CONF_PREFIX "/ui/browser_views"
+#define CONF_UI_SHOW_NOTIFICATIONS CONF_PREFIX "/ui/show_notifications"
 #define CONF_STATE_PLAY_ORDER	CONF_PREFIX "/state/play_order"
 #define CONF_STATE_VOLUME	CONF_PREFIX "/state/volume"
+#define CONF_STATE_BURN_DEVICE	CONF_PREFIX "/state/burn_device"
+
+#define CONF_AUDIOSCROBBLER_USERNAME CONF_PREFIX "/audioscrobbler/username"
+#define CONF_AUDIOSCROBBLER_PASSWORD CONF_PREFIX "/audioscrobbler/password"
+#define CONF_AUDIOSCROBBLER_ENABLED CONF_PREFIX "/audioscrobbler/enabled"
 
 G_END_DECLS
 
