@@ -1,4 +1,3 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
  /*
  *  arch-tag: Header for RhythmDB - Rhythmbox backend queryable database
  *
@@ -48,6 +47,8 @@ typedef gint32 RhythmDBEntryType;
 
 #define RHYTHMDB_ENTRY_TYPE_SONG (rhythmdb_entry_song_get_type ())
 #define RHYTHMDB_ENTRY_TYPE_IRADIO_STATION (rhythmdb_entry_iradio_get_type ())
+#define RHYTHMDB_ENTRY_TYPE_PODCAST_POST (rhythmdb_entry_podcast_post_get_type ())
+#define RHYTHMDB_ENTRY_TYPE_PODCAST_FEED (rhythmdb_entry_podcast_feed_get_type ())
 
 typedef enum
 {
@@ -59,6 +60,11 @@ typedef enum
 	RHYTHMDB_QUERY_PROP_NOT_LIKE,
 	RHYTHMDB_QUERY_PROP_GREATER,
 	RHYTHMDB_QUERY_PROP_LESS,
+	RHYTHMDB_QUERY_PROP_CURRENT_TIME_WITHIN,
+	RHYTHMDB_QUERY_PROP_CURRENT_TIME_NOT_WITHIN,
+	RHYTHMDB_QUERY_PROP_YEAR_EQUALS,
+	RHYTHMDB_QUERY_PROP_YEAR_GREATER,   
+	RHYTHMDB_QUERY_PROP_YEAR_LESS,   
 } RhythmDBQueryType;
 
 /* If you modify this enum, don't forget to modify rhythmdb_prop_get_type */
@@ -79,10 +85,10 @@ typedef enum
 	RHYTHMDB_PROP_FIRST_SEEN,
 	RHYTHMDB_PROP_LAST_SEEN,
 	RHYTHMDB_PROP_RATING,
-	RHYTHMDB_PROP_AUTO_RATE,
 	RHYTHMDB_PROP_PLAY_COUNT,
 	RHYTHMDB_PROP_LAST_PLAYED,
 	RHYTHMDB_PROP_BITRATE,
+	RHYTHMDB_PROP_DATE,
 	RHYTHMDB_PROP_TRACK_GAIN,
 	RHYTHMDB_PROP_TRACK_PEAK,
 	RHYTHMDB_PROP_ALBUM_GAIN,
@@ -99,10 +105,28 @@ typedef enum
 	RHYTHMDB_PROP_LAST_PLAYED_STR,
 	RHYTHMDB_PROP_HIDDEN,
 	RHYTHMDB_PROP_PLAYBACK_ERROR,
+	RHYTHMDB_PROP_FIRST_SEEN_STR,
+	RHYTHMDB_PROP_SEARCH_MATCH,
 
+	/* Podcast properties */
+	RHYTHMDB_PROP_STATUS,
+	RHYTHMDB_PROP_DESCRIPTION,
+	RHYTHMDB_PROP_SUBTITLE,
+	RHYTHMDB_PROP_SUMMARY,
+	RHYTHMDB_PROP_LANG,
+	RHYTHMDB_PROP_COPYRIGHT,
+	RHYTHMDB_PROP_IMAGE,
+	RHYTHMDB_PROP_POST_TIME,
+	
 	RHYTHMDB_NUM_PROPERTIES
 } RhythmDBPropType;
 
+enum {
+	RHYTHMDB_PODCAST_STATUS_COMPLETE = 100,
+	RHYTHMDB_PODCAST_STATUS_ERROR = 101,
+	RHYTHMDB_PODCAST_STATUS_WAITING = 102,
+	RHYTHMDB_PODCAST_STATUS_PAUSED = 103,
+};
 
 GType rhythmdb_query_get_type (void);
 GType rhythmdb_prop_get_type (void);
@@ -117,11 +141,29 @@ typedef struct {
 	GPtrArray *subquery;
 } RhythmDBQueryData;
 
+typedef struct {
+	/* podcast */
+	RBRefString *description;
+	RBRefString *subtitle;
+	RBRefString *summary;
+	RBRefString *lang;
+	RBRefString *copyright;
+	RBRefString *image;
+	gulong status;	/* 0-99: downloading
+			   100: Complete
+			   101: Error
+			   102: wait
+			   103: pause */
+	gulong post_time;
+} RhythmDBPodcastFields;
+
+
 typedef struct {
 	/* internal bits */
 #ifndef G_DISABLE_ASSERT
 	guint magic;
 #endif	
+	gboolean inserted;
 	gint refcount;
 	void *data;
 	gulong type;
@@ -139,6 +181,7 @@ typedef struct {
 	double track_peak;
 	double album_gain;
 	double album_peak;
+	GDate *date;
 
 	/* filesystem */
 	char *location;
@@ -155,18 +198,21 @@ typedef struct {
 
 	/* user data */
 	gdouble rating;
-	gboolean auto_rate;
 	glong play_count;
-	glong last_played;
+	gulong last_played;
 
 	/* cached data */
 	RBRefString *last_played_str;
+	RBRefString *first_seen_str;
 
 	/* playback error string */
 	char *playback_error;
 
 	/* visibility (to hide entries on unmounted volumes) */
 	gboolean hidden;
+
+	/*Podcast*/
+	RhythmDBPodcastFields *podcast;
 } RhythmDBEntry;
 
 typedef struct {
@@ -175,6 +221,7 @@ typedef struct {
 	GValue new;
 } RhythmDBEntryChange;
 
+void rhythmdb_entry_get (RhythmDBEntry *entry, RhythmDBPropType propid, GValue *val);
 G_INLINE_FUNC const char *rhythmdb_entry_get_string	(RhythmDBEntry *entry, RhythmDBPropType propid);
 G_INLINE_FUNC gboolean rhythmdb_entry_get_boolean	(RhythmDBEntry *entry, RhythmDBPropType propid);
 G_INLINE_FUNC guint64 rhythmdb_entry_get_uint64		(RhythmDBEntry *entry, RhythmDBPropType propid);
@@ -186,8 +233,7 @@ G_INLINE_FUNC double rhythmdb_entry_get_double		(RhythmDBEntry *entry, RhythmDBP
 G_INLINE_FUNC const char *
 rhythmdb_entry_get_string (RhythmDBEntry *entry, RhythmDBPropType propid)
 {
-	switch (propid)
-	{
+	switch (propid) {
 	case RHYTHMDB_PROP_TITLE:
 		return rb_refstring_get (entry->title);
 	case RHYTHMDB_PROP_ALBUM:
@@ -222,6 +268,42 @@ rhythmdb_entry_get_string (RhythmDBEntry *entry, RhythmDBPropType propid)
 		return rb_refstring_get (entry->last_played_str);
 	case RHYTHMDB_PROP_PLAYBACK_ERROR:
 		return entry->playback_error;
+	case RHYTHMDB_PROP_FIRST_SEEN_STR:
+		return rb_refstring_get (entry->first_seen_str);
+	case RHYTHMDB_PROP_SEARCH_MATCH:
+		return NULL;	/* synthetic property */
+	/* Podcast properties */
+	case RHYTHMDB_PROP_DESCRIPTION:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->description);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_SUBTITLE:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->subtitle);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_SUMMARY: 
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->summary);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_LANG:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->lang);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_COPYRIGHT:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->copyright);
+		else
+			return NULL;
+	case RHYTHMDB_PROP_IMAGE:
+		if (entry->podcast)
+			return rb_refstring_get (entry->podcast->image);
+		else
+			return NULL;
+
 	default:
 		g_assert_not_reached ();
 		return NULL;
@@ -232,8 +314,6 @@ G_INLINE_FUNC gboolean
 rhythmdb_entry_get_boolean (RhythmDBEntry *entry, RhythmDBPropType propid)
 {
 	switch (propid) {
-	case RHYTHMDB_PROP_AUTO_RATE:
-		return entry->auto_rate;
 	case RHYTHMDB_PROP_HIDDEN:
 		return entry->hidden;
 	default:
@@ -278,6 +358,21 @@ rhythmdb_entry_get_ulong (RhythmDBEntry *entry, RhythmDBPropType propid)
 		return entry->play_count;
 	case RHYTHMDB_PROP_BITRATE:
 		return entry->bitrate;		
+	case RHYTHMDB_PROP_DATE:
+		if (entry->date)
+			return g_date_get_julian (entry->date);
+		else
+			return 0;
+	case RHYTHMDB_PROP_POST_TIME:
+		if (entry->podcast)
+			return entry->podcast->post_time;
+		else
+			return 0;
+	case RHYTHMDB_PROP_STATUS:
+		if (entry->podcast)
+			return entry->podcast->status;		
+		else
+			return 0;
 	default:
 		g_assert_not_reached ();
 		return 0;
@@ -334,7 +429,8 @@ typedef struct
 	void	(*entry_deleted)	(RhythmDBEntry *entry);
 	void	(*load_complete)	(void);
 	void	(*save_complete)	(void);
-	void	(*error)		(const char *uri, const char *msg);
+	void	(*load_error)		(const char *uri, const char *msg);
+	void	(*save_error)		(const char *uri, const GError *error);
 	void	(*read_only)		(gboolean readonly);
 
 	/* virtual methods */
@@ -344,7 +440,7 @@ typedef struct
 	
 	void		(*impl_entry_new)	(RhythmDB *db, RhythmDBEntry *entry);
 
-	void		(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
+	gboolean	(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
 	                                 guint propid, const GValue *value);
 
 	void		(*impl_entry_get)	(RhythmDB *db, RhythmDBEntry *entry,
@@ -355,7 +451,7 @@ typedef struct
 	void            (*impl_entry_delete_by_type) (RhythmDB *db, RhythmDBEntryType type);
 
 	RhythmDBEntry *	(*impl_lookup_by_location)(RhythmDB *db, const char *uri);
-
+	
 	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, GPtrArray *query, RhythmDBEntry *entry);
 
 	void		(*impl_entry_foreach)	(RhythmDB *db, GFunc func, gpointer data);
@@ -383,17 +479,20 @@ gboolean	rhythmdb_entry_is_editable (RhythmDB *db, RhythmDBEntry *entry);
 RhythmDBEntry *	rhythmdb_entry_new	(RhythmDB *db, RhythmDBEntryType type, const char *uri);
 
 void		rhythmdb_add_uri	(RhythmDB *db, const char *uri);
+void		rhythmdb_add_uri_with_type	(RhythmDB *db, const char *uri, RhythmDBEntryType type);
 
 void		rhythmdb_entry_set	(RhythmDB *db, RhythmDBEntry *entry,
-					 guint propid, GValue *value);
-void            rhythmdb_entry_sync     (RhythmDB *db, RhythmDBEntry *entry,
-					 guint propid, GValue *value);
+					 guint propid, const GValue *value);
+void		rhythmdb_entry_set_nonotify	(RhythmDB *db, RhythmDBEntry *entry,
+						 guint propid, const GValue *value);
+void		rhythmdb_entry_set_uninserted   (RhythmDB *db, RhythmDBEntry *entry,
+						 guint propid, const GValue *value);
 
 void		rhythmdb_entry_delete	(RhythmDB *db, RhythmDBEntry *entry);
 void            rhythmdb_entry_delete_by_type (RhythmDB *db, 
 					       RhythmDBEntryType type);
-
-void		rhythmdb_entry_destroy	(RhythmDB *db, RhythmDBEntry *entry);
+void		rhythmdb_entry_move_to_trash (RhythmDB *db,
+					      RhythmDBEntry *entry);
 
 
 RhythmDBEntry *	rhythmdb_entry_lookup_by_location (RhythmDB *db, const char *uri);
@@ -442,6 +541,7 @@ GPtrArray *	rhythmdb_query_parse			(RhythmDB *db, ...);
 void		rhythmdb_query_append			(RhythmDB *db, GPtrArray *query, ...);
 void		rhythmdb_query_free			(GPtrArray *query);
 GPtrArray *	rhythmdb_query_copy			(GPtrArray *array);
+void		rhythmdb_query_preprocess		(RhythmDB *db, GPtrArray *query);
 
 void		rhythmdb_query_serialize		(RhythmDB *db, GPtrArray *query,
 							 xmlNodePtr node);
@@ -461,6 +561,8 @@ RhythmDBEntryType rhythmdb_entry_register_type          (void);
 
 RhythmDBEntryType rhythmdb_entry_song_get_type          (void);
 RhythmDBEntryType rhythmdb_entry_iradio_get_type        (void);
+RhythmDBEntryType rhythmdb_entry_podcast_post_get_type  (void);
+RhythmDBEntryType rhythmdb_entry_podcast_feed_get_type  (void);
 RhythmDBEntryType rhythmdb_entry_icecast_get_type        (void);
 
 extern GType rhythmdb_property_type_map[RHYTHMDB_NUM_PROPERTIES];
