@@ -1,8 +1,9 @@
-/* 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ * 
  *  arch-tag: Implementation of search entry/browse toggle container
  *
- *  Copyright (C) 2003 Colin Walters <walters@debian.org>
  *  Copyright (C) 2003 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@redhat.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,23 +21,22 @@
  *
  */
 
-#include <gtk/gtk.h>
 #include <config.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
-#include <bonobo/bonobo-ui-util.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 
 #include "disclosure-widget.h"
 #include "rb-source-header.h"
 #include "rb-stock-icons.h"
-#include "rb-thread-helpers.h"
-#include "rb-bonobo-helpers.h"
 #include "rb-preferences.h"
 #include "rb-search-entry.h"
 #include "rb-debug.h"
 #include "rb-remote.h"
+#include "rb-entry-view.h"
 #include "eel-gconf-extensions.h"
 
 static void rb_source_header_class_init (RBSourceHeaderClass *klass);
@@ -55,27 +55,20 @@ static void rb_source_header_filter_changed_cb (RBSource *source,
 static void rb_source_header_search_cb (RBSearchEntry *search,
 					const char *text,
 					RBSourceHeader *header);
-static void rb_source_header_disclosure_toggled_cb (GtkToggleButton *disclosure,
+static void rb_source_header_disclosure_toggled_cb (GObject *object,
+						    GParamSpec *param_spec,
 						    gpointer data);
-static void rb_source_header_gconf_search_text_changed_cb (GConfClient *client,
-							   guint cnxn_id,
-							   GConfEntry *entry,
-							   RBSourceHeader *header);
-static void rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
-							  guint cnxn_id,
-							  GConfEntry *entry,
-							  RBSourceHeader *header);
-static void rb_source_header_view_browser_changed_cb (BonoboUIComponent *component,
-						      const char *path,
-						      Bonobo_UIComponent_EventType type,
-						      const char *state,
+static void rb_source_header_search_activate_cb (RBSearchEntry *search,
+						 RBSourceHeader *header);
+static void rb_source_header_view_browser_changed_cb (GtkAction *action,
 						      RBSourceHeader *header);
+static void rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source);
 
 struct RBSourceHeaderPrivate
 {
 	RBSource *selected_source;
 
-	BonoboUIComponent *component;
+	GtkActionGroup *actiongroup;
 
 	GtkTooltips *tooltips;
 
@@ -84,63 +77,39 @@ struct RBSourceHeaderPrivate
 
 	guint browser_notify_id;
 	guint search_notify_id;
+	gboolean have_search;
+	gboolean have_browser;
+	gboolean disclosed;
 	const char *browser_key;
-	const char *search_key;
+
+	GHashTable *source_search_text;
 };
 
+#define RB_SOURCE_HEADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCE_HEADER, RBSourceHeaderPrivate))
+
 enum
 {
 	PROP_0,
-	PROP_COMPONENT,
+	PROP_ACTION_GROUP,
 	PROP_SOURCE,
 };
 
-#define CMD_PATH_VIEW_BROWSER	"/commands/ViewBrowser"
-#define CMD_PATH_VIEW_STATUSBAR  "/commands/ViewStatusbar"
-
-static RBBonoboUIListener rb_source_header_listeners[] =
+static GtkToggleActionEntry rb_source_header_toggle_entries [] =
 {
-	RB_BONOBO_UI_LISTENER ("ViewBrowser", (BonoboUIListenerFn) rb_source_header_view_browser_changed_cb),
-	RB_BONOBO_UI_LISTENER_END
+	{ "ViewBrowser", NULL, N_("_Browser"), "<control>B",
+	  N_("Change the visibility of the browser"),
+	  G_CALLBACK (rb_source_header_view_browser_changed_cb) }
 };
+static guint rb_source_header_n_toggle_entries = G_N_ELEMENTS (rb_source_header_toggle_entries);
 
-static GObjectClass *parent_class = NULL;
 
-GType
-rb_source_header_get_type (void)
-{
-	static GType rb_source_header_type = 0;
-
-	if (rb_source_header_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBSourceHeaderClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_source_header_class_init,
-			NULL,
-			NULL,
-			sizeof (RBSourceHeader),
-			0,
-			(GInstanceInitFunc) rb_source_header_init
-		};
-
-		rb_source_header_type = g_type_register_static (GTK_TYPE_TABLE,
-							       "RBSourceHeader",
-							       &our_info, 0);
-	}
-
-	return rb_source_header_type;
-}
+G_DEFINE_TYPE (RBSourceHeader, rb_source_header, GTK_TYPE_TABLE)
 
 static void
 rb_source_header_class_init (RBSourceHeaderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_source_header_finalize;
 
 	object_class->set_property = rb_source_header_set_property;
@@ -154,12 +123,14 @@ rb_source_header_class_init (RBSourceHeaderClass *klass)
 							      RB_TYPE_SOURCE,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
-					 PROP_COMPONENT,
-					 g_param_spec_object ("component",
-							      "BonoboUIComponent",
-							      "BonoboUIComponent object",
-							      BONOBO_TYPE_UI_COMPONENT,
+					 PROP_ACTION_GROUP,
+					 g_param_spec_object ("action-group",
+							      "GtkActionGroup",
+							      "GtkActionGroup object",
+							      GTK_TYPE_ACTION_GROUP,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (klass, sizeof (RBSourceHeaderPrivate));
 }
 
 static void
@@ -167,7 +138,8 @@ rb_source_header_init (RBSourceHeader *header)
 {
 	GtkWidget *align;
 	GtkEventBox *ebox;
-	header->priv = g_new0 (RBSourceHeaderPrivate, 1);
+
+	header->priv = RB_SOURCE_HEADER_GET_PRIVATE (header);
 
 	header->priv->tooltips = gtk_tooltips_new ();
 	gtk_tooltips_enable (header->priv->tooltips);
@@ -186,11 +158,13 @@ rb_source_header_init (RBSourceHeader *header)
 
 	g_signal_connect_object (G_OBJECT (header->priv->search), "search",
 				 G_CALLBACK (rb_source_header_search_cb), header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->search), "activate",
+				 G_CALLBACK (rb_source_header_search_activate_cb), header, 0);
 
 	header->priv->disclosure = cddb_disclosure_new (_("Show _Browser"),
 							_("Hide _Browser"));
 	gtk_widget_set_sensitive (header->priv->disclosure, FALSE);
-	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "toggled",
+	g_signal_connect_object (G_OBJECT (header->priv->disclosure), "notify::expanded",
 				 G_CALLBACK (rb_source_header_disclosure_toggled_cb), header, 0);
 
 	gtk_table_attach_defaults (GTK_TABLE (header),
@@ -198,8 +172,22 @@ rb_source_header_init (RBSourceHeader *header)
 
 	align = gtk_alignment_new (1.0, 0.5, 1.0, 1.0);
 	gtk_container_add (GTK_CONTAINER (align), GTK_WIDGET (ebox));
-	gtk_table_attach_defaults (GTK_TABLE (header),
-			           align, 2, 3, 0, 1);
+	gtk_table_attach (GTK_TABLE (header),
+			  align, 2, 3, 0, 1, 
+			  GTK_EXPAND | GTK_FILL,
+			  GTK_EXPAND | GTK_FILL,
+			  5, 0);
+
+	header->priv->source_search_text = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+								  NULL, (GDestroyNotify)g_free);
+}
+
+static void
+rb_source_header_source_weak_unref (RBSource *source, char *text, RBSourceHeader *header)
+{
+	g_object_weak_unref (G_OBJECT (source),
+			     (GWeakNotify)rb_source_header_source_weak_destroy_cb,
+			     header);
 }
 
 static void
@@ -214,9 +202,12 @@ rb_source_header_finalize (GObject *object)
 
 	g_return_if_fail (header->priv != NULL);
 
-	g_free (header->priv);
+	g_hash_table_foreach (header->priv->source_search_text,
+			      (GHFunc) rb_source_header_source_weak_unref,
+			      header);
+	g_hash_table_destroy (header->priv->source_search_text);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_source_header_parent_class)->finalize (object);
 }
 
 static void
@@ -232,11 +223,6 @@ rb_source_header_set_property (GObject *object,
 	case PROP_SOURCE:
 		if (header->priv->selected_source != NULL)
 		{
-			if (header->priv->browser_key)
-				eel_gconf_notification_remove (header->priv->browser_notify_id);
-			if (header->priv->search_key)
-				eel_gconf_notification_remove (header->priv->search_notify_id);
-
 			g_signal_handlers_disconnect_by_func (G_OBJECT (header->priv->selected_source),
 							      G_CALLBACK (rb_source_header_filter_changed_cb),
 							      header);
@@ -245,47 +231,43 @@ rb_source_header_set_property (GObject *object,
 		header->priv->selected_source = g_value_get_object (value);
 		rb_debug ("selected source %p", g_value_get_object (value));
 
-		if (header->priv->selected_source != NULL)
-		{
+		if (header->priv->selected_source != NULL) {
+			const char *text = g_hash_table_lookup (header->priv->source_search_text,
+								header->priv->selected_source);
+			
 			header->priv->browser_key = rb_source_get_browser_key (header->priv->selected_source);
-			header->priv->search_key = rb_source_get_search_key (header->priv->selected_source);
-			if (header->priv->browser_key)
-				header->priv->browser_notify_id
-					= eel_gconf_notification_add (header->priv->browser_key,
-								      (GConfClientNotifyFunc) rb_source_header_gconf_disclosure_changed_cb,
-								      header);
-			if (header->priv->search_key) {
-				gchar *temp = eel_gconf_get_string (header->priv->search_key);
-				
-				header->priv->search_notify_id
-					= eel_gconf_notification_add (header->priv->search_key,
-								      (GConfClientNotifyFunc) rb_source_header_gconf_search_text_changed_cb,
-								      header);
-
-				rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search),
-							  temp);
-				g_free (temp);
-			}
+	
+			rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search), text);
 			g_signal_connect_object (G_OBJECT (header->priv->selected_source),
 						 "filter_changed",
 						 G_CALLBACK (rb_source_header_filter_changed_cb),
 						 header, 0);
+	
 			gtk_widget_set_sensitive (GTK_WIDGET (header->priv->search),
 						  rb_source_can_search (header->priv->selected_source));
-			if (!header->priv->browser_key &&
-			    !rb_source_can_search (header->priv->selected_source))
+			header->priv->have_search = rb_source_can_search (header->priv->selected_source);
+			header->priv->have_browser = rb_source_can_browse (header->priv->selected_source);
+			if (!header->priv->have_browser)
+				header->priv->disclosed = FALSE;
+			else if (header->priv->browser_key)
+				header->priv->disclosed = eel_gconf_get_boolean (header->priv->browser_key);
+			else
+				/* FIXME: remember the previous state of the source*/
+				header->priv->disclosed = FALSE;
+	
+			if (!header->priv->have_browser && !header->priv->have_search)
 				gtk_widget_hide (GTK_WIDGET (header));
 			else
 				gtk_widget_show (GTK_WIDGET (header));
 		}
 		rb_source_header_sync_control_state (header);
-		
 		break;
-	case PROP_COMPONENT:
-		header->priv->component = g_value_get_object (value);
-		rb_bonobo_add_listener_list_with_data (header->priv->component,
-						       rb_source_header_listeners,
-						       header);
+	case PROP_ACTION_GROUP:
+		header->priv->actiongroup = g_value_get_object (value);
+		gtk_action_group_add_toggle_actions (header->priv->actiongroup,
+						     rb_source_header_toggle_entries,
+						     rb_source_header_n_toggle_entries,
+						     header);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -306,8 +288,8 @@ rb_source_header_get_property (GObject *object,
 	case PROP_SOURCE:
 		g_value_set_object (value, header->priv->selected_source);
 		break;
-	case PROP_COMPONENT:
-		g_value_set_object (value, header->priv->component);
+	case PROP_ACTION_GROUP:
+		g_value_set_object (value, header->priv->actiongroup);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -328,10 +310,10 @@ rb_source_header_set_source (RBSourceHeader *header,
 }
 
 RBSourceHeader *
-rb_source_header_new (BonoboUIComponent *component)
+rb_source_header_new (GtkActionGroup *actiongroup)
 {
 	RBSourceHeader *header = g_object_new (RB_TYPE_SOURCE_HEADER,
-					       "component", component,
+					       "action-group", actiongroup,
 					       NULL);
 
 	g_return_val_if_fail (header->priv != NULL, NULL);
@@ -353,25 +335,9 @@ rb_source_header_filter_changed_cb (RBSource *source,
 }
 
 static void
-rb_source_header_gconf_search_text_changed_cb (GConfClient *client,
-					       guint cnxn_id,
-					       GConfEntry *entry,
-					       RBSourceHeader *header)
+rb_source_header_source_weak_destroy_cb (RBSourceHeader *header, RBSource *source)
 {
-	char *searchtext;
-
-	g_return_if_fail (header->priv->search_key != NULL);
-	
-	rb_debug ("gconf search text changed");
-
-	searchtext = eel_gconf_get_string (header->priv->search_key);
-
-	rb_search_entry_set_text (RB_SEARCH_ENTRY (header->priv->search),
-				  searchtext);
-
-	rb_source_search (header->priv->selected_source, searchtext);
-
-	g_free (searchtext);
+	g_hash_table_remove (header->priv->source_search_text, source);
 }
 
 static void
@@ -381,7 +347,18 @@ rb_source_header_search_cb (RBSearchEntry *search,
 {
 	rb_debug  ("searching for \"%s\"", text);
 
-	eel_gconf_set_string (header->priv->search_key, text);
+	/* if we haven't seen the source before, monitor it for deletion */
+	if (g_hash_table_lookup (header->priv->source_search_text, header->priv->selected_source) == NULL) {
+		g_object_weak_ref (G_OBJECT (header->priv->selected_source),
+				   (GWeakNotify)rb_source_header_source_weak_destroy_cb,
+				   header);
+	}
+
+	g_hash_table_insert (header->priv->source_search_text,
+			     header->priv->selected_source,
+			     g_strdup (text));
+	rb_source_search (header->priv->selected_source, text);
+	rb_source_header_sync_control_state (header);
 }
 
 void
@@ -395,65 +372,70 @@ rb_source_header_clear_search (RBSourceHeader *header)
 	if (header->priv->selected_source)
 		rb_source_search (header->priv->selected_source, NULL);
 	rb_search_entry_clear (RB_SEARCH_ENTRY (header->priv->search));
+	rb_source_header_sync_control_state (header);
 }
 
 static void
-rb_source_header_disclosure_toggled_cb (GtkToggleButton *disclosure,
+rb_source_header_disclosure_toggled_cb (GObject *object,
+					GParamSpec *param_spec,
 					gpointer data)
 {
 	RBSourceHeader *header = RB_SOURCE_HEADER (data);
-	gboolean disclosed = gtk_toggle_button_get_active (disclosure);
+	GtkExpander *expander = GTK_EXPANDER (object);
 
-	rb_debug ("disclosed: %s", disclosed ? "TRUE" : "FALSE");
-	
-	if (header->priv->selected_source != NULL)
-		eel_gconf_set_boolean (header->priv->browser_key, disclosed);
-}
+	header->priv->disclosed = gtk_expander_get_expanded (expander);
 
-static void
-rb_source_header_gconf_disclosure_changed_cb (GConfClient *client,
-					      guint cnxn_id,
-					      GConfEntry *entry,
-					      RBSourceHeader *header)
-{
-	rb_debug ("gconf disclosure changed");
-
-	g_return_if_fail (header->priv->browser_key != NULL);
+	if (header->priv->browser_key)
+		eel_gconf_set_boolean (header->priv->browser_key, 
+				       header->priv->disclosed);
 
 	rb_source_header_sync_control_state (header);
 }
 
 static void
-rb_source_header_view_browser_changed_cb (BonoboUIComponent *component,
-					  const char *path,
-					  Bonobo_UIComponent_EventType type,
-					  const char *state,
+rb_source_header_view_browser_changed_cb (GtkAction *action,
 					  RBSourceHeader *header)
 {
 	rb_debug ("got view browser toggle");
-	eel_gconf_set_boolean (header->priv->browser_key,
-			       rb_bonobo_get_active (component, CMD_PATH_VIEW_BROWSER));
+	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
+				   gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
 void
 rb_source_header_sync_control_state (RBSourceHeader *header)
 {
-	gboolean have_browser = header->priv->selected_source != NULL
-		&& header->priv->browser_key != NULL;
+	GtkAction *viewbrowser_action;
+	GtkAction *viewstatusbar_action;
+	GtkAction *viewall_action;
 	gboolean not_small = !eel_gconf_get_boolean (CONF_UI_SMALL_DISPLAY);
 
 	gtk_widget_set_sensitive (header->priv->disclosure,
-				  have_browser);
-	rb_bonobo_set_sensitive (header->priv->component, CMD_PATH_VIEW_BROWSER,
-				have_browser && not_small);
- 	rb_bonobo_set_sensitive (header->priv->component, 
- 				CMD_PATH_VIEW_STATUSBAR, not_small);
-	if (have_browser) {
-		gboolean shown = eel_gconf_get_boolean (header->priv->browser_key);
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (header->priv->disclosure),
-					      shown);
-		rb_bonobo_set_active (header->priv->component,
-				      CMD_PATH_VIEW_BROWSER,
-				      shown);
-	}
+				  header->priv->have_browser);
+	viewbrowser_action = gtk_action_group_get_action (header->priv->actiongroup,
+							  "ViewBrowser");
+	g_object_set (G_OBJECT (viewbrowser_action), "sensitive",
+		      header->priv->have_browser && not_small, NULL);
+	viewstatusbar_action = gtk_action_group_get_action (header->priv->actiongroup,
+							    "ViewStatusbar");
+	g_object_set (G_OBJECT (viewstatusbar_action), "sensitive",
+		      not_small, NULL);
+	viewall_action = gtk_action_group_get_action (header->priv->actiongroup,
+						      "ViewAll");
+	g_object_set (G_OBJECT (viewall_action), "sensitive",
+		      header->priv->have_browser && not_small, NULL);
+
+	gtk_expander_set_expanded (GTK_EXPANDER (header->priv->disclosure),
+				   header->priv->disclosed);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (viewbrowser_action),
+				      header->priv->disclosed);
+}
+
+static void
+rb_source_header_search_activate_cb (RBSearchEntry *search,
+				     RBSourceHeader *header)
+{
+	RBEntryView *entry_view;
+
+	entry_view = rb_source_get_entry_view (header->priv->selected_source);
+	gtk_widget_grab_focus (GTK_WIDGET (entry_view));
 }
