@@ -35,7 +35,11 @@
 #include "rb-stock-icons.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
+#include "rb-util.h"
 
+static GObject *rb_removable_media_source_constructor (GType type,
+						       guint n_construct_properties,
+						       GObjectConstructParam *construct_properties);
 static void rb_removable_media_source_dispose (GObject *object);
 
 static void rb_removable_media_source_set_property (GObject *object,
@@ -72,13 +76,28 @@ rb_removable_media_source_class_init (RBRemovableMediaSourceClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
+	RBLibrarySourceClass *library_source_class = RB_LIBRARY_SOURCE_CLASS (klass);
 
+	object_class->constructor = rb_removable_media_source_constructor;
 	object_class->dispose = rb_removable_media_source_dispose;
 	object_class->set_property = rb_removable_media_source_set_property;
 	object_class->get_property = rb_removable_media_source_get_property;
 
 	source_class->impl_get_pixbuf = impl_get_pixbuf;
 	source_class->impl_delete_thyself = impl_delete_thyself;
+	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_receive_drag = NULL;
+	source_class->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
+	source_class->impl_paste = NULL;
+	source_class->impl_delete = NULL;
+	source_class->impl_get_config_widget = NULL;
+	source_class->impl_show_popup = (RBSourceFeatureFunc) rb_false_function;
+
+	library_source_class->impl_get_paned_key = NULL;
+	library_source_class->impl_has_first_added_column = (RBLibrarySourceFeatureFunc) rb_false_function;
+	library_source_class->impl_has_drop_support = (RBLibrarySourceFeatureFunc) rb_false_function;
 
 	g_object_class_install_property (object_class,
 					 PROP_VOLUME,
@@ -97,6 +116,29 @@ rb_removable_media_source_init (RBRemovableMediaSource *self)
 
 }
 
+static GObject *
+rb_removable_media_source_constructor (GType type, guint n_construct_properties,
+				       GObjectConstructParam *construct_properties)
+{
+	GObject *source; 
+	GnomeVFSVolume *volume;
+	char *display_name;
+
+	source = G_OBJECT_CLASS(rb_removable_media_source_parent_class)
+			->constructor (type, n_construct_properties, construct_properties);
+
+	g_object_get (source, "volume", &volume, NULL);
+	
+	display_name = gnome_vfs_volume_get_display_name (volume);
+	g_object_set (source, "name", display_name, NULL);
+	g_free (display_name);
+
+	g_object_unref (volume);
+
+	return source;
+}
+
+
 static void 
 rb_removable_media_source_dispose (GObject *object)
 {
