@@ -32,6 +32,7 @@
 #include "rb-shell.h"
 #include "rb-source.h"
 #include "rb-util.h"
+#include "rb-static-playlist-source.h"
 
 static void rb_source_class_init (RBSourceClass *klass);
 static void rb_source_init (RBSource *source);
@@ -45,25 +46,31 @@ static void rb_source_get_property (GObject *object,
 					GValue *value,
 					GParamSpec *pspec);
 
-static const char * default_get_browser_key (RBSource *status);
+static const char * default_get_browser_key (RBSource *source);
 static GList *default_get_extra_views (RBSource *source);
 static gboolean default_can_rename (RBSource *source);
 static gboolean default_can_search (RBSource *source);
-static gboolean default_can_cut (RBSource *source);
 static GdkPixbuf *default_get_pixbuf (RBSource *source);
 static GList *default_copy (RBSource *source);
 static void default_reset_filters (RBSource *source);
 static void default_song_properties (RBSource *source);
-static GtkWidget * default_get_config_widget (RBSource *source);
 static gboolean default_try_playlist (RBSource *source);
 static RBSourceEOFType default_handle_eos (RBSource *source);
-static gboolean default_receive_drag  (RBSource *source, GtkSelectionData *data);
 static gboolean default_show_popup  (RBSource *source);
 static void default_delete_thyself (RBSource *source);
 static void default_activate (RBSource *source);
 static void default_deactivate (RBSource *source);
 static gboolean default_disconnect (RBSource *source);
+static void default_add_to_queue (RBSource *source, RBSource *queue);
+static char *default_get_status (RBSource *source);
 
+static void rb_source_row_deleted_cb (GtkTreeModel *model,
+				      GtkTreePath *path,
+				      RBSource *source);
+static void rb_source_row_inserted_cb (GtkTreeModel *model,
+				       GtkTreePath *path,
+				       GtkTreeIter *iter,
+				       RBSource *source);
 G_DEFINE_ABSTRACT_TYPE (RBSource, rb_source, GTK_TYPE_HBOX)
 #define RB_SOURCE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), RB_TYPE_SOURCE, RBSourcePrivate))
 
@@ -73,6 +80,8 @@ struct _RBSourcePrivate
 	
 	RBShell *shell;
 	gboolean visible;
+	RhythmDBQueryModel *query_model;
+	guint idle_status_changed_id;
 };
 
 enum
@@ -81,7 +90,8 @@ enum
 	PROP_NAME,
 	PROP_SHELL,
 	PROP_UI_MANAGER,
-	PROP_VISIBLE
+	PROP_VISIBLE,
+	PROP_QUERY_MODEL
 };
 
 enum
@@ -104,26 +114,31 @@ rb_source_class_init (RBSourceClass *klass)
 	object_class->set_property = rb_source_set_property;
 	object_class->get_property = rb_source_get_property;
 
+	klass->impl_can_browse = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_get_browser_key = default_get_browser_key;
 	klass->impl_get_extra_views = default_get_extra_views;
 	klass->impl_can_rename = default_can_rename;
 	klass->impl_can_search = default_can_search;
-	klass->impl_can_cut = default_can_cut;
-	klass->impl_can_delete = default_can_cut;
-	klass->impl_can_copy = default_can_cut;
+	klass->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
+	klass->impl_can_delete = (RBSourceFeatureFunc) rb_false_function;
+	klass->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
+	klass->impl_can_add_to_queue = (RBSourceFeatureFunc) rb_false_function;
+	klass->impl_can_move_to_trash = (RBSourceFeatureFunc) rb_false_function;
 	klass->impl_get_pixbuf = default_get_pixbuf;
 	klass->impl_copy = default_copy;
 	klass->impl_reset_filters = default_reset_filters;
 	klass->impl_song_properties = default_song_properties;
 	klass->impl_handle_eos = default_handle_eos;
-	klass->impl_get_config_widget = default_get_config_widget;
-	klass->impl_receive_drag = default_receive_drag;
+	klass->impl_get_config_widget = NULL;
+	klass->impl_receive_drag = NULL;
 	klass->impl_show_popup = default_show_popup;
 	klass->impl_delete_thyself = default_delete_thyself;
 	klass->impl_activate = default_activate;
 	klass->impl_deactivate = default_deactivate;
 	klass->impl_disconnect = default_disconnect;
 	klass->impl_try_playlist = default_try_playlist;
+	klass->impl_add_to_queue = default_add_to_queue;
+	klass->impl_get_status = default_get_status;
 
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
@@ -162,7 +177,15 @@ rb_source_class_init (RBSourceClass *klass)
 							       "visibility",
 							       "Whether the source should be displayed in the source list",
 							       TRUE,
-							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+							       G_PARAM_READWRITE));
+
+	g_object_class_install_property (object_class,
+					 PROP_QUERY_MODEL,
+					 g_param_spec_object ("query-model",
+						 	      "RhythmDBQueryModel",
+							      "RhythmDBQueryModel object",
+							      RHYTHMDB_TYPE_QUERY_MODEL,
+							      G_PARAM_READWRITE));
 
 	rb_source_signals[DELETED] =
 		g_signal_new ("deleted",
@@ -223,6 +246,11 @@ rb_source_finalize (GObject *object)
 
 	rb_debug ("Finalizing view %p", source);
 
+	if (priv->idle_status_changed_id)
+		g_source_remove (priv->idle_status_changed_id);
+	
+	g_object_unref (G_OBJECT (priv->query_model));
+	
 	g_free (priv->name);
 
 	G_OBJECT_CLASS (rb_source_parent_class)->finalize (object);
@@ -235,6 +263,8 @@ rb_source_set_property (GObject *object,
 		      GParamSpec *pspec)
 {
 	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (object);
+	RhythmDBQueryModel *model;
+	RBSource *source = RB_SOURCE (object);
 
 	switch (prop_id)
 	{
@@ -251,6 +281,35 @@ rb_source_set_property (GObject *object,
 			  priv->name, 
 			  priv->visible);
 		break;
+	case PROP_QUERY_MODEL:
+		model = g_value_get_object (value); 
+		if (priv->query_model == model)
+			return;
+
+		if (priv->query_model) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (model),
+							      G_CALLBACK (rb_source_row_deleted_cb),
+							      source);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (model),
+							      G_CALLBACK (rb_source_row_inserted_cb),
+							      source);
+			g_object_unref (G_OBJECT (priv->query_model));
+		}
+		
+		priv->query_model = model;
+		if (priv->query_model) {
+			g_object_ref (G_OBJECT (model));
+			g_signal_connect_object (G_OBJECT (model), "row_deleted",
+						 G_CALLBACK (rb_source_row_deleted_cb),
+						 source, 0);
+			g_signal_connect_object (G_OBJECT (model), "row_inserted",
+						 G_CALLBACK (rb_source_row_inserted_cb),
+						 source, 0);
+		}
+
+		/* g_object_notify (G_OBJECT (source), "query-model"); */
+		rb_source_notify_status_changed (source);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -286,12 +345,25 @@ rb_source_get_property (GObject *object,
 		g_object_unref (G_OBJECT(manager));
 		break;
 	}
+	case PROP_QUERY_MODEL:
+		g_value_set_object (value, priv->query_model);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
 	}
 }
 
+static char *
+default_get_status (RBSource *source)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+	if (priv->query_model)
+		return rhythmdb_query_model_compute_status_normal (priv->query_model);
+
+	return g_strdup ("");
+}
+
 /**
  * rb_source_get_status:
  * @status: a #RBSource
@@ -302,37 +374,45 @@ rb_source_get_property (GObject *object,
  * Returns: The status string
  **/
 char *
-rb_source_get_status (RBSource *status)
+rb_source_get_status (RBSource *source)
 {
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (status);
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_get_status (status);
+	return klass->impl_get_status (source);
 }
 
 static const char *
-default_get_browser_key (RBSource *status)
+default_get_browser_key (RBSource *source)
 {
 	return NULL;
 }
 
 const char *
-rb_source_get_browser_key (RBSource *status)
+rb_source_get_browser_key (RBSource *source)
 {
-	RBSourceClass *klass = RB_SOURCE_GET_CLASS (status);
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_get_browser_key (status);
+	return klass->impl_get_browser_key (source);
+}
+
+gboolean
+rb_source_can_browse (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return klass->impl_can_browse (source);
 }
 
 void
-rb_source_notify_status_changed (RBSource *status)
+rb_source_notify_status_changed (RBSource *source)
 {
-	g_signal_emit (G_OBJECT (status), rb_source_signals[STATUS_CHANGED], 0);
+	g_signal_emit (G_OBJECT (source), rb_source_signals[STATUS_CHANGED], 0);
 }
 
 void
-rb_source_notify_filter_changed (RBSource *status)
+rb_source_notify_filter_changed (RBSource *source)
 {
-	g_signal_emit (G_OBJECT (status), rb_source_signals[FILTER_CHANGED], 0);
+	g_signal_emit (G_OBJECT (source), rb_source_signals[FILTER_CHANGED], 0);
 }
 
 void
@@ -437,30 +517,18 @@ rb_source_search (RBSource *source, const char *text)
 		klass->impl_search (source, text);
 }
 
-static GtkWidget *
-default_get_config_widget (RBSource *source)
-{
-	return NULL;
-}
-
 GtkWidget *
-rb_source_get_config_widget (RBSource *source)
+rb_source_get_config_widget (RBSource *source, RBShellPreferences *prefs)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
 	if (klass->impl_get_config_widget) {
-		return klass->impl_get_config_widget (source);
+		return klass->impl_get_config_widget (source, prefs);
 	} else {
 		return NULL;
 	}
 }
 
-static gboolean
-default_can_cut (RBSource *source)
-{
-	return FALSE;
-}
-
 gboolean
 rb_source_can_cut (RBSource *source)
 {
@@ -477,6 +545,14 @@ rb_source_can_delete (RBSource *source)
 	return klass->impl_can_delete (source);
 }
 
+gboolean
+rb_source_can_move_to_trash (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	return klass->impl_can_move_to_trash (source);
+}
+
 gboolean
 rb_source_can_copy (RBSource *source)
 {
@@ -515,6 +591,38 @@ rb_source_paste (RBSource *source, GList *nodes)
 	klass->impl_paste (source, nodes);
 }
 
+gboolean
+rb_source_can_add_to_queue (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	return klass->impl_can_add_to_queue (source);
+}
+
+static void
+default_add_to_queue (RBSource *source, RBSource *queue)
+{
+	RBEntryView *songs = rb_source_get_entry_view (source);
+	GList *selection = rb_entry_view_get_selected_entries (songs);
+	GList *iter;
+
+	if (selection == NULL) 
+		return;
+
+	for (iter = selection; iter; iter = iter->next) {
+		rb_static_playlist_source_add_entry (RB_STATIC_PLAYLIST_SOURCE (queue), 
+						     (RhythmDBEntry *)iter->data, -1);
+	}
+
+	g_list_free (selection);
+}
+
+void
+rb_source_add_to_queue (RBSource *source, RBSource *queue)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+	klass->impl_add_to_queue (source, queue);
+}
+
 void
 rb_source_delete (RBSource *source)
 {
@@ -523,6 +631,14 @@ rb_source_delete (RBSource *source)
 	klass->impl_delete (source);
 }
 
+void
+rb_source_move_to_trash (RBSource *source)
+{
+	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
+
+	klass->impl_move_to_trash (source);
+}
+
 static void
 default_reset_filters (RBSource *source)
 {
@@ -595,21 +711,15 @@ rb_source_have_url (RBSource *source)
 	return klass->impl_have_url (source);
 }
 
-gboolean
-default_receive_drag (RBSource *source, GtkSelectionData *data)
-{
-	rb_error_dialog (NULL,
-			 _("Not supported"),
-			 _("This source does not support drag and drop."));
-	return FALSE;
-}
-
 gboolean
 rb_source_receive_drag (RBSource *source, GtkSelectionData *data)
 {
 	RBSourceClass *klass = RB_SOURCE_GET_CLASS (source);
 
-	return klass->impl_receive_drag (source, data);
+	if (klass->impl_receive_drag)
+		return klass->impl_receive_drag (source, data);
+	else
+		return FALSE;
 }
 
 void
@@ -693,3 +803,37 @@ gboolean rb_source_disconnect (RBSource *source)
 	return klass->impl_disconnect (source);
 }
 
+static void
+rb_source_row_inserted_cb (GtkTreeModel *model,
+			   GtkTreePath *path,
+			   GtkTreeIter *iter,
+			   RBSource *source)
+{
+	rb_source_notify_status_changed (source);
+}
+
+static gboolean
+idle_emit_status_changed (RBSource *source)
+{
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+	rb_source_notify_status_changed (source);
+	priv->idle_status_changed_id = 0;
+	return FALSE;
+}
+
+static void
+rb_source_row_deleted_cb (GtkTreeModel *model,
+			  GtkTreePath *path,
+			  RBSource *source)
+{
+	/* Emit the signal after the deletion has actually been processed,
+	 * since if we update the status now it'll show the old state.
+	 */
+	RBSourcePrivate *priv = RB_SOURCE_GET_PRIVATE (source);
+	if (priv->idle_status_changed_id == 0) {
+		priv->idle_status_changed_id = 
+			g_idle_add ((GSourceFunc) idle_emit_status_changed, source);
+	}
+}
+
+
