@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  * arch-tag: Implementation of main "Sources" display widget
  *
  * Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
@@ -22,12 +23,13 @@
 
 #include "config.h"
 
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-#include <libgnome/gnome-i18n.h>
 #include <unistd.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+#include <gdk/gdk.h>
+#include <gtk/gtk.h>
+
 #include "rb-sourcelist.h"
 #include "rb-sourcelist-model.h"
 #include "rb-debug.h"
@@ -37,7 +39,7 @@
 #include "rb-tree-dnd.h"
 #include "rb-util.h"
 
-struct RBSourceListPriv
+struct RBSourceListPrivate
 {
 	GtkWidget *treeview;
 	GtkCellRenderer *title_renderer;
@@ -51,9 +53,13 @@ struct RBSourceListPriv
 	GtkTreeViewColumn *hidden_column;
 	GtkTreeViewColumn *main_column;
 
+	RBSource *preempted_source;
+
 	RBShell *shell;
 };
 
+#define RB_SOURCELIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_SOURCELIST, RBSourceListPrivate))
+
 enum
 {
 	PROP_0,
@@ -183,6 +189,8 @@ rb_sourcelist_class_init (RBSourceListClass *class)
 			      G_TYPE_BOOLEAN,
 			      1,
 			      G_TYPE_POINTER);
+
+	g_type_class_add_private (class, sizeof (RBSourceListPrivate));
 }
 
 static void
@@ -190,7 +198,7 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 {
 	GtkCellRenderer *renderer;
 
-	sourcelist->priv = g_new0 (RBSourceListPriv, 1);
+	sourcelist->priv = RB_SOURCELIST_GET_PRIVATE (sourcelist);
 
 	sourcelist->priv->filter_model = rb_sourcelist_model_new ();
 	sourcelist->priv->real_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (sourcelist->priv->filter_model));
@@ -241,6 +249,9 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 
 	/* Set up the name column */
 	sourcelist->priv->title_renderer = renderer = gtk_cell_renderer_text_new ();
+	g_object_set (G_OBJECT (sourcelist->priv->title_renderer),
+		      "ellipsize", PANGO_ELLIPSIZE_END,
+		      NULL);
 	gtk_tree_view_column_pack_start (sourcelist->priv->main_column, renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (sourcelist->priv->main_column, renderer,
 						 (GtkTreeCellDataFunc)
@@ -261,12 +272,9 @@ rb_sourcelist_init (RBSourceList *sourcelist)
 static void
 rb_sourcelist_finalize (GObject *object)
 {
-	RBSourceList *sourcelist = RB_SOURCELIST (object);
-
-	g_free (sourcelist->priv);
-
 	G_OBJECT_CLASS (rb_sourcelist_parent_class)->finalize (object);
 }
+
 static void
 rb_sourcelist_set_property (GObject *object,
 			    guint prop_id,
@@ -454,22 +462,44 @@ rb_sourcelist_edit_source_name (RBSourceList *sourcelist, RBSource *source)
 	gtk_tree_path_free (path);
 }
 
+static void set_source_playing (RBSourceList *sourcelist, RBSource *source, gboolean playing)
+{
+	GtkTreeIter iter;
+	g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
+	gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
+			    RB_SOURCELIST_MODEL_COLUMN_PLAYING, playing, -1);
+}
+
 void
 rb_sourcelist_set_playing_source (RBSourceList *sourcelist, RBSource *source)
 {
-	GtkTreeIter iter;
-	if (sourcelist->priv->playing_source) {
-		g_assert (rb_sourcelist_source_to_iter (sourcelist,
-							sourcelist->priv->playing_source,
-							&iter));
-		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, FALSE, -1);
-	}
+	if (sourcelist->priv->playing_source)
+		set_source_playing (sourcelist, sourcelist->priv->playing_source, FALSE);
+
 	sourcelist->priv->playing_source = source;
-	if (source) {
-		g_assert (rb_sourcelist_source_to_iter (sourcelist, source, &iter));
-		gtk_tree_store_set (GTK_TREE_STORE (sourcelist->priv->real_model), &iter,
-				    RB_SOURCELIST_MODEL_COLUMN_PLAYING, TRUE, -1);
+	if (source)
+		set_source_playing (sourcelist, source, TRUE);
+}
+
+void
+rb_sourcelist_preempt_playing_source (RBSourceList *sourcelist, RBSource *source)
+{
+	if (source == NULL) {
+		if (sourcelist->priv->preempted_source) {
+			set_source_playing (sourcelist, sourcelist->priv->playing_source, FALSE);
+			set_source_playing (sourcelist, sourcelist->priv->preempted_source, TRUE);
+
+			sourcelist->priv->playing_source = sourcelist->priv->preempted_source;
+			sourcelist->priv->preempted_source = NULL;
+		}
+	} else {
+		if (sourcelist->priv->playing_source) {
+			set_source_playing (sourcelist, sourcelist->priv->playing_source, FALSE);
+			set_source_playing (sourcelist, source, TRUE);
+
+			sourcelist->priv->preempted_source = sourcelist->priv->playing_source;
+			sourcelist->priv->playing_source = source;
+		}
 	}
 }
 
