@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of main song information display widget
  *
  *  Copyright (C) 2002, 2003 Jorn Baayen <jorn@nl.linux.org>
@@ -20,24 +21,19 @@
  *
  */
 
-#include <math.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkhscale.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktooltips.h>
 #include <config.h>
-#include <libgnome/gnome-i18n.h>
+
+#include <math.h>
 #include <string.h>
 
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <libgnomeui/libgnomeui.h> /* for GnomeUrl */
+
 #include "rb-song-display-box.h"
 #include "rb-stock-icons.h"
 #include "rb-header.h"
 #include "rb-debug.h"
-#include "rb-ellipsizing-label.h"
 #include "rb-preferences.h"
 #include "eel-gconf-extensions.h"
 
@@ -55,13 +51,14 @@ static void rb_header_get_property (GObject *object,
 static long rb_header_get_duration (RBHeader *player);
 static void rb_header_set_show_timeline (RBHeader *player,
 			                 gboolean show);
-static gboolean rb_header_sync_time_locked (RBHeader *player);
 static void rb_header_update_elapsed (RBHeader *player);
 static gboolean slider_press_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *player);
 static gboolean slider_moved_callback (GtkWidget *widget, GdkEventMotion *event, RBHeader *player);
 static gboolean slider_release_callback (GtkWidget *widget, GdkEventButton *event, RBHeader *player);
 static void slider_changed_callback (GtkWidget *widget, RBHeader *player);
 
+static void rb_header_tick_cb (RBPlayer *mmplayer, long time, RBHeader *header);
+
 typedef struct
 {
 	long elapsed;
@@ -77,13 +74,15 @@ struct RBHeaderPrivate
 	char *urltext, *urllink;
 
 	RBPlayer *mmplayer;
+	gulong tick_id;
 
 	GtkWidget *image;
 	GtkWidget *song;
 
-	GtkWidget *timeframe;
 	GtkWidget *timeline;
-	gboolean timeline_shown;
+	GtkWidget *scaleline;
+	gboolean scaleline_shown;
+
 	GtkWidget *scale;
 	GtkAdjustment *adjustment;
 	gboolean slider_dragging;
@@ -93,69 +92,38 @@ struct RBHeaderPrivate
 	guint value_changed_update_handler;
 	GtkWidget *elapsed;
 
-	GtkWidget *textframe;
 	RBSongDisplayBox *displaybox;
 	gboolean displaybox_shown;
-	GtkTooltips *tips;
-	GtkWidget *urlframe;
 	GtkWidget *urlline;
 	gboolean urlline_shown;
 	GnomeHRef *url;
 
-	guint timeout;
-	
 	RBHeaderState *state;
 };
 
+#define RB_HEADER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_HEADER, RBHeaderPrivate))
+
 enum
 {
 	PROP_0,
 	PROP_DB,
-	PROP_HEADER,
+	PROP_PLAYER,
 	PROP_ENTRY,
 	PROP_TITLE,
 	PROP_URLTEXT,
 	PROP_URLLINK,
 };
 
-static GObjectClass *parent_class = NULL;
-
 #define SONG_MARKUP(xSONG) g_strdup_printf ("<big><b>%s</b></big>", xSONG);
 
-GType
-rb_header_get_type (void)
-{
-	static GType rb_header_type = 0;
+G_DEFINE_TYPE (RBHeader, rb_header, GTK_TYPE_HBOX)
 
-	if (rb_header_type == 0) {
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBHeaderClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_header_class_init,
-			NULL,
-			NULL,
-			sizeof (RBHeader),
-			0,
-			(GInstanceInitFunc) rb_header_init
-		};
-
-		rb_header_type = g_type_register_static (GTK_TYPE_HBOX,
-							 "RBHeader",
-							 &our_info, 0);
-	}
-
-	return rb_header_type;
-}
 
 static void
 rb_header_class_init (RBHeaderClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_header_finalize;
 
 	object_class->set_property = rb_header_set_property;
@@ -176,10 +144,10 @@ rb_header_class_init (RBHeaderClass *klass)
 							       "RhythmDBEntry pointer",
 							       G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
-					 PROP_HEADER,
+					 PROP_PLAYER,
 					 g_param_spec_object ("player",
 							      "Player",
-							      "MonkeyMediaPlayer",
+							      "RBPlayer object",
 							      RB_TYPE_PLAYER,
 							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
@@ -204,122 +172,110 @@ rb_header_class_init (RBHeaderClass *klass)
 							      NULL,
 							      G_PARAM_READWRITE));
 
+	g_type_class_add_private (klass, sizeof (RBHeaderPrivate));
 }
 
 static void
-rb_header_init (RBHeader *player)
+rb_header_init (RBHeader *header)
 {
 	/**
 	 * The children in this widget look like this:
 	 * RBHeader
 	 *   GtkHBox
-	 *     GtkVBox
-	 *       GtkAlignment
-	 *         GtkVBox
-	 *           RBEllipsizingLabel	(priv->song)
-	 *           GtkHBox		(priv->textframe)
-	 *	       RBSongDisplayBox (priv->displaybox)
-	 *               GtkLabel	("from")
-	 *               GnomeHRef	(priv->displaybox->album)
-	 *               GtkLabel	("by")
-	 *               GnomeHRef	(priv->displaybox->artist)
-	 *           GtkHBox		(priv->urlframe)
-	 *	       GtkHBox		(priv->urlline)
-	 *               GtkLabel	("Listening to")
-	 *               GnomeHRef	(priv->url)
-	 *   GtkAlignment		(priv->timeframe)
-	 *     GtkVBox			(priv->timeline)
-	 *       GtkHScale		(priv->scale)
-	 *       GtkAlignment
-	 *         GtkLabel		(priv->elapsed)
+	 *     GtkLabel			(priv->song)
+	 *     RBSongDisplayBox		(priv->displaybox)
+	 *       GtkLabel		("from")
+	 *       GtkWidget		(priv->displaybox->album)
+	 *       GtkLabel		("by")
+	 *       GtkWidget		(priv->displaybox->artist)
+	 *     GtkHBox			(priv->urlline)
+	 *       GtkLabel		("Listening to")
+	 *       GnomeHRef		(priv->url)
+	 *   GtkHBox			(priv->timeline)
+	 *     GtkHScale		(priv->scale)
+	 *     GtkAlignment
+	 *       GtkLabel		(priv->elapsed)
 	 */
-	GtkWidget *hbox, *vbox, *urlline, *label, *align, *scalebox, *textvbox;
+	GtkWidget *urlline, *label,  *hbox;
+	GtkWidget *vbox;
+	char *s;
 
-	player->priv = g_new0 (RBHeaderPrivate, 1);
+	header->priv = g_new0 (RBHeaderPrivate, 1);
+	header->priv->state = g_new0 (RBHeaderState, 1);
 
-	player->priv->state = g_new0 (RBHeaderState, 1);
+	gtk_box_set_spacing (GTK_BOX (header), 3);
 
-	hbox = gtk_hbox_new (FALSE, 10);
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_widget_show (vbox);
+	gtk_box_pack_start (GTK_BOX (header), vbox, TRUE, TRUE, 0);
 
-#if 0
-	player->priv->image = gtk_image_new ();
-	gtk_box_pack_start (GTK_BOX (hbox), player->priv->image, FALSE, TRUE, 0);
-#endif
+	/* song info */
+	hbox = gtk_hbox_new (FALSE, 16);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+	gtk_widget_show (hbox);
 
-	vbox = gtk_vbox_new (FALSE, 5);
-
-	align = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
-	textvbox = gtk_vbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (align), textvbox);
-	
-	player->priv->song = rb_ellipsizing_label_new ("");
-	rb_ellipsizing_label_set_mode (RB_ELLIPSIZING_LABEL (player->priv->song), RB_ELLIPSIZE_END);
- 	gtk_label_set_use_markup (GTK_LABEL (player->priv->song), TRUE);
- 	gtk_label_set_selectable (GTK_LABEL (player->priv->song), TRUE);	
-	gtk_misc_set_alignment (GTK_MISC (player->priv->song), 0, 0);
-	gtk_box_pack_start (GTK_BOX (textvbox), player->priv->song, FALSE, TRUE, 0);
+	header->priv->song = gtk_label_new ("");
+ 	gtk_label_set_use_markup (GTK_LABEL (header->priv->song), TRUE);
+ 	gtk_label_set_selectable (GTK_LABEL (header->priv->song), TRUE);	
+	gtk_box_pack_start (GTK_BOX (hbox), header->priv->song, FALSE, TRUE, 0);
+	gtk_widget_show (header->priv->song);
 
 	/* Construct the Artist/Album display */
-	player->priv->textframe = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (textvbox), player->priv->textframe, FALSE, TRUE, 0);
-
-	player->priv->displaybox = RB_SONG_DISPLAY_BOX (rb_song_display_box_new ());
-	g_object_ref (G_OBJECT (player->priv->displaybox));
-	player->priv->displaybox_shown = FALSE;
+	header->priv->displaybox = RB_SONG_DISPLAY_BOX (rb_song_display_box_new ());
+	header->priv->displaybox_shown = FALSE;
+	gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (header->priv->displaybox), TRUE, TRUE, 0);
 
 	/* Construct the URL display */
-	player->priv->urlframe = gtk_hbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (textvbox), player->priv->urlframe, FALSE, TRUE, 0);
 
-	urlline = player->priv->urlline = gtk_hbox_new (FALSE, 2);
-	g_object_ref (G_OBJECT (urlline));
+	urlline = header->priv->urlline = gtk_hbox_new (FALSE, 2);
+	gtk_box_pack_start (GTK_BOX (hbox), header->priv->urlline, TRUE, TRUE, 0);
 
 	label = gtk_label_new (_("Listening to "));
 	gtk_box_pack_start (GTK_BOX (urlline), GTK_WIDGET (label), FALSE, TRUE, 0);
-	player->priv->url = (GnomeHRef *) gnome_href_new ("", "");
-	gtk_box_pack_start (GTK_BOX (urlline), GTK_WIDGET (player->priv->url), FALSE, TRUE, 0);
-	player->priv->urlline_shown = FALSE;
+	header->priv->url = (GnomeHRef *) gnome_href_new ("", "");
+	gtk_box_pack_start (GTK_BOX (urlline), GTK_WIDGET (header->priv->url), FALSE, TRUE, 0);
+	header->priv->urlline_shown = FALSE;
 
-	/* construct the time slider and display */
-	scalebox = player->priv->timeline = gtk_vbox_new (FALSE, 0);
-	g_object_ref (G_OBJECT (scalebox));
+	/* construct the time display */
+	header->priv->timeline = gtk_hbox_new (FALSE, 3);
 
-	player->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 10.0, 0.0));
-	player->priv->scale = gtk_hscale_new (player->priv->adjustment);
-	g_signal_connect_object (G_OBJECT (player->priv->scale),
+	s = rb_header_get_elapsed_string (header);
+	header->priv->elapsed = gtk_label_new (s);
+	g_free (s);
+
+	gtk_misc_set_padding (GTK_MISC (header->priv->elapsed), 2, 0);
+	gtk_box_pack_start (GTK_BOX (header->priv->timeline), header->priv->elapsed, FALSE, FALSE, 0);
+	gtk_widget_set_sensitive (header->priv->timeline, FALSE);
+	gtk_box_pack_end (GTK_BOX (hbox), header->priv->timeline, FALSE, FALSE, 0);
+	gtk_widget_show_all (header->priv->timeline);
+
+	/* row for the position slider */
+	header->priv->scaleline = gtk_hbox_new (FALSE, 3);
+	gtk_box_pack_start (GTK_BOX (vbox), header->priv->scaleline, FALSE, FALSE, 0);
+	header->priv->scaleline_shown = FALSE;
+
+	header->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 10.0, 0.0));
+	header->priv->scale = gtk_hscale_new (header->priv->adjustment);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
 				 "button_press_event",
 				 G_CALLBACK (slider_press_callback),
-				 player, 0);
-	g_signal_connect_object (G_OBJECT (player->priv->scale),
+				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
 				 "button_release_event",
 				 G_CALLBACK (slider_release_callback),
-				 player, 0);
-	g_signal_connect_object (G_OBJECT (player->priv->scale),
+				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
 				 "motion_notify_event",
 				 G_CALLBACK (slider_moved_callback),
-				 player, 0);
-	g_signal_connect_object (G_OBJECT (player->priv->scale),
+				 header, 0);
+	g_signal_connect_object (G_OBJECT (header->priv->scale),
 				 "value_changed",
 				 G_CALLBACK (slider_changed_callback),
-				 player, 0);
-	gtk_scale_set_draw_value (GTK_SCALE (player->priv->scale), FALSE);
-	gtk_widget_set_size_request (player->priv->scale, 150, -1);
-	gtk_box_pack_start (GTK_BOX (scalebox), player->priv->scale, FALSE, TRUE, 0);
-	align = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
-	player->priv->elapsed = gtk_label_new ("0:00");
-	gtk_misc_set_padding (GTK_MISC (player->priv->elapsed), 2, 0);
-	player->priv->tips = gtk_tooltips_new ();
-	gtk_container_add (GTK_CONTAINER (align), player->priv->elapsed);
-	gtk_box_pack_start (GTK_BOX (scalebox), align, FALSE, TRUE, 0);
-	align = player->priv->timeframe = gtk_alignment_new (1.0, 0.5, 0.0, 0.0);
+				 header, 0);
+	gtk_scale_set_draw_value (GTK_SCALE (header->priv->scale), FALSE);
+	gtk_widget_set_size_request (header->priv->scale, 150, -1);
+	gtk_box_pack_start (GTK_BOX (header->priv->scaleline), header->priv->scale, TRUE, TRUE, 0);
 
-	gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
-
-	gtk_box_pack_start (GTK_BOX (player), hbox, TRUE, TRUE, 0);
-	gtk_box_pack_end (GTK_BOX (player), align, FALSE, FALSE, 0);
-
-	player->priv->timeout = g_timeout_add (1000, (GSourceFunc) rb_header_sync_time_locked, player);
 }
 
 static void
@@ -331,19 +287,12 @@ rb_header_finalize (GObject *object)
 	g_return_if_fail (RB_IS_HEADER (object));
 
 	player = RB_HEADER (object);
-
 	g_return_if_fail (player->priv != NULL);
 
-	g_source_remove (player->priv->timeout);
-
-	g_object_unref (G_OBJECT (player->priv->urlline));
-	g_object_unref (G_OBJECT (player->priv->displaybox));
-	g_object_unref (G_OBJECT (player->priv->timeline));
-
 	g_free (player->priv->state);
 	g_free (player->priv);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_header_parent_class)->finalize (object);
 }
 
 static void
@@ -361,8 +310,17 @@ rb_header_set_property (GObject *object,
 	case PROP_ENTRY:
 		player->priv->entry = g_value_get_pointer (value);
 		break;
-	case PROP_HEADER:
+	case PROP_PLAYER:
+		if (player->priv->mmplayer) {
+			g_signal_handler_disconnect (player->priv->mmplayer,
+						     player->priv->tick_id);
+			player->priv->tick_id = 0;
+		}
 		player->priv->mmplayer = g_value_get_object (value);
+		player->priv->tick_id = g_signal_connect (G_OBJECT (player->priv->mmplayer),
+							  "tick", 
+							  (GCallback) rb_header_tick_cb,
+							  player);
 		break;
 	case PROP_TITLE:
 		g_free (player->priv->title);
@@ -397,7 +355,7 @@ rb_header_get_property (GObject *object,
 	case PROP_ENTRY:
 		g_value_set_object (value, player->priv->entry);
 		break;
-	case PROP_HEADER:
+	case PROP_PLAYER:
 		g_value_set_object (value, player->priv->mmplayer);
 		break;
 	case PROP_TITLE:
@@ -444,124 +402,38 @@ rb_header_set_title (RBHeader *player, const char *title)
 static long
 rb_header_get_duration (RBHeader *player)
 {
-	long ret;
 	if (player->priv->entry) {
-		rhythmdb_read_lock (player->priv->db);
-		ret = rhythmdb_entry_get_long (player->priv->db,
-					       player->priv->entry,
-					       RHYTHMDB_PROP_DURATION);
-		rhythmdb_read_unlock (player->priv->db);
-		return ret;
+		return player->priv->entry->duration;
 	}
 	return -1;
 }
 
-static char *
-info_url (guint opt, const char *data)
-{
-	char * ret, *p;
-
-	ret = g_strdup_printf ("http://www.allmusic.com/cg/amg.dll?p=amg&opt1=%d&sql=%s", opt, data);
-
-	for (p = ret; *p; p++)
-		if (*p == ' ')
-			*p = '+';
-	return ret;
-}
-
-static char *
-album_info_url (const char *album)
-{
-	return info_url (2, album);
-}
-
-static char *
-artist_info_url (const char *album)
-{
-	return info_url (1, album);
-}
-
 void
 rb_header_sync (RBHeader *player)
 {
 	char *tmp;
 
 	rb_debug ("syncing with node = %p", player->priv->entry);
+	rb_song_display_box_sync (player->priv->displaybox, player->priv->entry);
+	
 	if (player->priv->entry != NULL) {
 		const char *song = player->priv->title;
-		char *escaped, *s;
+		char *escaped;
 		gboolean have_duration = rb_header_get_duration (player) > 0;
-		const char *album; 
-		const char *artist; 
-		GtkTooltips *artist_href_tips, *album_href_tips;
+		const char *album, *artist; 
 
-		rhythmdb_read_lock (player->priv->db);
-
-		album = rhythmdb_entry_get_string (player->priv->db,
-						   player->priv->entry,
-						   RHYTHMDB_PROP_ALBUM);
-
-		artist = rhythmdb_entry_get_string (player->priv->db,
-						    player->priv->entry,
-						    RHYTHMDB_PROP_ARTIST);
-
-		rhythmdb_read_unlock (player->priv->db);
+		album = rhythmdb_entry_get_string (player->priv->entry, RHYTHMDB_PROP_ALBUM);
+		artist = rhythmdb_entry_get_string (player->priv->entry, RHYTHMDB_PROP_ARTIST);
 
 		escaped = g_markup_escape_text (song, -1);
 		tmp = SONG_MARKUP (escaped);
 		g_free (escaped);
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (player->priv->song), tmp);
+		gtk_label_set_markup (GTK_LABEL (player->priv->song), tmp);
 		g_free (tmp);
 
 		rb_header_set_show_artist_album (player, (album != NULL && artist != NULL)
 						 && (strlen (album) > 0 && strlen (artist) > 0));
 
-		if (player->priv->displaybox_shown)
-		{
-			g_return_if_fail (album != NULL);
-			g_return_if_fail (artist != NULL);
-
-			rb_debug ("artist: %s album: %s", artist, album);
-
-			s = tmp = g_strdup (album);
-			while ((tmp = strstr (tmp, " ")) != NULL)
-				*tmp = '|';
-			tmp = album_info_url (s);
-			g_free (s);
-
-			gnome_href_set_url (player->priv->displaybox->album, tmp);
-			escaped = g_markup_escape_text (album, -1);
-			gnome_href_set_text (player->priv->displaybox->album, escaped);
-			g_free (escaped);
-			g_free (tmp);
-   
-   			album_href_tips = gtk_tooltips_new ();
-			gtk_tooltips_set_tip (GTK_TOOLTIPS (album_href_tips), 
-					      GTK_WIDGET (player->priv->displaybox->album),
-					      _("Get information on this album from the web"), 
-					      NULL);
-			
-			s = tmp = g_strdup (artist);
-			while ((tmp = strstr (tmp, " ")) != NULL)
-			{
-				*tmp = '|';
-			}
-			tmp = artist_info_url (s);
-			g_free (s);
-			gnome_href_set_url (player->priv->displaybox->artist, tmp);
-			escaped = g_markup_escape_text (artist, -1);
-			gnome_href_set_text (player->priv->displaybox->artist, escaped);
-			g_free (escaped);
-			g_free (tmp);
-
-   			artist_href_tips = gtk_tooltips_new ();
-			gtk_tooltips_set_tip (GTK_TOOLTIPS (artist_href_tips), 
-					      GTK_WIDGET (player->priv->displaybox->artist),
-					      _("Get information on this artist from the web"),
-					      NULL);
-			
-		}
-
 		if (player->priv->urlline_shown)
 		{
 			g_return_if_fail (player->priv->urltext != NULL);
@@ -575,30 +447,24 @@ rb_header_sync (RBHeader *player)
 		if (have_duration)
 			rb_header_sync_time (player);
 	} else {
-		GtkTooltips *iradio_href_tips;
-
 		rb_debug ("not playing");
-		tmp = SONG_MARKUP (_("Not playing"));
-		rb_ellipsizing_label_set_markup (RB_ELLIPSIZING_LABEL (player->priv->song), tmp);
+		tmp = SONG_MARKUP (_("Not Playing"));
+		gtk_label_set_markup (GTK_LABEL (player->priv->song), tmp);
 		g_free (tmp);
 
-		iradio_href_tips = gtk_tooltips_new ();
-		gtk_tooltips_set_tip (GTK_TOOLTIPS (iradio_href_tips), 
-				      GTK_WIDGET (player->priv->displaybox->artist),
-				      _("Get information on this station from the web"),
-				      NULL);
-
 		rb_header_set_urldata (player, NULL, NULL);
 		rb_header_set_show_artist_album (player, FALSE);
 		rb_header_set_show_timeline (player, FALSE);
-	}
 
-#if 0
-	if (pixbuf != NULL)
-		gtk_image_set_from_pixbuf (GTK_IMAGE (player->priv->image), pixbuf);
-	else
-		gtk_image_set_from_stock (GTK_IMAGE (player->priv->image), RB_STOCK_ALBUM, GTK_ICON_SIZE_DIALOG);
-#endif
+		player->priv->slider_locked = TRUE;
+		gtk_adjustment_set_value (player->priv->adjustment, 0.0);
+		player->priv->slider_locked = FALSE;
+		gtk_widget_set_sensitive (player->priv->scale, FALSE);
+
+		tmp = rb_header_get_elapsed_string (player);
+		gtk_label_set_text (GTK_LABEL (player->priv->elapsed), tmp);
+		g_free (tmp);
+	}
 }
 
 void
@@ -614,13 +480,10 @@ rb_header_set_urldata (RBHeader *player,
 	g_object_set (G_OBJECT (player), "urltext", urltext,
 		      "urllink", urllink, NULL);
 
-	if (show == FALSE)
-		gtk_container_remove (GTK_CONTAINER (player->priv->urlframe), player->priv->urlline);
-	else
-	{
-		gtk_container_add (GTK_CONTAINER (player->priv->urlframe), player->priv->urlline);
+	if (show)
 		gtk_widget_show_all (player->priv->urlline);
-	}
+	else
+		gtk_widget_hide (player->priv->urlline);
 }
 
 void
@@ -632,11 +495,26 @@ rb_header_set_show_artist_album (RBHeader *player,
 
 	player->priv->displaybox_shown = show;
 
-	if (show == FALSE)
-		gtk_container_remove (GTK_CONTAINER (player->priv->textframe), GTK_WIDGET (player->priv->displaybox));
-	else {
-		gtk_container_add (GTK_CONTAINER (player->priv->textframe), GTK_WIDGET (player->priv->displaybox));
-		gtk_widget_show_all (player->priv->textframe);
+	if (show)
+		gtk_widget_show_all (GTK_WIDGET (player->priv->displaybox));
+	else
+		gtk_widget_hide (GTK_WIDGET (player->priv->displaybox));
+}
+
+void
+rb_header_set_show_position_slider (RBHeader *player,
+				    gboolean show)
+{
+	if (player->priv->scaleline_shown == show)
+		return;
+
+	player->priv->scaleline_shown = show;
+
+	if (show) {
+		gtk_widget_show_all (GTK_WIDGET (player->priv->scaleline));
+		rb_header_sync_time (player);
+	} else {
+		gtk_widget_hide (GTK_WIDGET (player->priv->scaleline));
 	}
 }
 
@@ -644,31 +522,8 @@ static void
 rb_header_set_show_timeline (RBHeader *player,
 			     gboolean show)
 {
-	if (player->priv->timeline_shown == show)
-		return;
-
-	player->priv->timeline_shown = show;
-
-	if (show == FALSE)
-		gtk_container_remove (GTK_CONTAINER (player->priv->timeframe), player->priv->timeline);
-	else {
-		gtk_container_add (GTK_CONTAINER (player->priv->timeframe), player->priv->timeline);
-		gtk_widget_show_all (player->priv->timeline);
-	}
-}
-
-gboolean
-rb_header_sync_time_locked (RBHeader *player)
-{
-	gboolean ret;
-
-	GDK_THREADS_ENTER ();
-
-	ret = rb_header_sync_time (player);
-
-	GDK_THREADS_LEAVE ();
-
-	return ret;
+	gtk_widget_set_sensitive (player->priv->timeline, show);
+	gtk_widget_set_sensitive (player->priv->scaleline, show);
 }
 
 gboolean
@@ -687,8 +542,7 @@ rb_header_sync_time (RBHeader *player)
 
 	player->priv->state->duration = duration =
 		rb_header_get_duration (player);
-	player->priv->state->elapsed = seconds =
-		rb_player_get_time (player->priv->mmplayer);
+	seconds = player->priv->state->elapsed;
 
 	if (duration > -1) {
 		double progress = 0.0;
@@ -847,30 +701,55 @@ slider_changed_callback (GtkWidget *widget,
 char *
 rb_header_get_elapsed_string (RBHeader *player)
 {
-	int seconds = 0, minutes = 0, seconds2 = -1, minutes2 = -1;
-	guint elapsed = rb_player_get_time (player->priv->mmplayer);
+	int seconds = 0, minutes = 0, hours = 0, seconds2 = -1, minutes2 = -1, hours2 = -1;
 
-	if (elapsed > 0) {
-		minutes = elapsed / 60;
-		seconds = elapsed % 60;
+	if (player->priv->state->elapsed > 0) {
+		hours = player->priv->state->elapsed / (60 * 60);
+		minutes = (player->priv->state->elapsed - (hours * 60 * 60)) / 60;
+		seconds = player->priv->state->elapsed % 60;
 	}
 
-	if (player->priv->state->duration > 0) {
-		minutes2 = player->priv->state->duration / 60;
-		seconds2 = player->priv->state->duration % 60;
-		if (eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY)) {
-			return g_strdup_printf (_("%d:%02d of %d:%02d"), minutes, seconds, minutes2, seconds2);
+	/* use this if entry==NULL, so that it doesn't shift when you first start playback */
+	if (player->priv->entry == NULL || player->priv->state->duration > 0) {
+		if (player->priv->state->duration > 0) {
+			hours2 = player->priv->state->duration / (60 * 60);
+			minutes2 = (player->priv->state->duration - (hours2 * 60 * 60)) / 60;
+			seconds2 = player->priv->state->duration % 60;
 		} else {
-			int remaining = player->priv->state->duration - elapsed;
-			int remaining_minutes = remaining / 60;
+			hours2 = minutes2 = seconds2 = 0;
+		}
+
+		if (eel_gconf_get_boolean (CONF_UI_TIME_DISPLAY)) {
+			if (hours == 0 && hours2 == 0)
+				return g_strdup_printf (_("%d:%02d of %d:%02d"),
+							minutes, seconds,
+							minutes2, seconds2);
+			else
+				return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d"),
+							hours, minutes, seconds,
+							hours2, minutes2, seconds2);
+		} else {
+			int remaining = player->priv->state->duration - player->priv->state->elapsed;
+			int remaining_hours = remaining / (60 * 60);
+			int remaining_minutes = (remaining - (remaining_hours * 60 * 60)) / 60;
 			/* remaining could conceivably be negative. This would
 			 * be a bug, but the elapsed time will display right
 			 * with the abs(). */
 			int remaining_seconds = abs (remaining % 60);
-			return g_strdup_printf (_("%d:%02d of %d:%02d remaining"), remaining_minutes, remaining_seconds, minutes2, seconds2);
+			if (hours2 == 0)
+				return g_strdup_printf (_("%d:%02d of %d:%02d remaining"),
+							remaining_minutes, remaining_seconds,
+							minutes2, seconds2);
+			else
+				return g_strdup_printf (_("%d:%02d:%02d of %d:%02d:%02d remaining"),
+							remaining_hours, remaining_minutes, remaining_seconds,
+							hours2, minutes2, seconds2);
 		}
 	} else {
-		return g_strdup_printf (_("%d:%02d"), minutes, seconds);
+		if (hours == 0)
+			return g_strdup_printf (_("%d:%02d"), minutes, seconds);
+		else
+			return g_strdup_printf (_("%d:%02d:%02d"), hours, minutes, seconds);
 	}
 }
 
@@ -888,3 +767,17 @@ rb_header_update_elapsed (RBHeader *player)
 	gtk_label_set_text (GTK_LABEL (player->priv->elapsed), elapsed_text);
 	g_free (elapsed_text);
 }
+
+static void rb_header_tick_cb (RBPlayer *mmplayer, long elapsed, RBHeader *header)
+{
+	gboolean sync = (elapsed != header->priv->state->elapsed);
+	header->priv->state->elapsed = elapsed;
+	
+	if (sync) {
+		GDK_THREADS_ENTER ();
+
+		rb_header_sync_time (header);
+
+		GDK_THREADS_LEAVE ();
+	}
+}
