@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *  arch-tag: Implementation of dialog that displays errors during library loading
  *
  *  Copyright (C) 2003 Colin Walters <walters@gnu.org>
@@ -20,10 +21,11 @@
  */
 
 #include <config.h>
+
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnome/gnome-i18n.h>
-#include <gtk/gtk.h>
 
 #include "rb-load-failure-dialog.h"
 #include "rb-dialog.h"
@@ -56,44 +58,18 @@ struct RBLoadFailureDialogPrivate
 	GtkListStore *liststore;
 };
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (RBLoadFailureDialog, rb_load_failure_dialog, GTK_TYPE_DIALOG)
+#define RB_LOAD_FAILURE_DIALOG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_LOAD_FAILURE_DIALOG, RBLoadFailureDialogPrivate))
 
-GType
-rb_load_failure_dialog_get_type (void)
-{
-	static GType rb_load_failure_dialog_type = 0;
-
-	if (rb_load_failure_dialog_type == 0)
-	{
-		static const GTypeInfo our_info =
-		{
-			sizeof (RBLoadFailureDialogClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) rb_load_failure_dialog_class_init,
-			NULL,
-			NULL,
-			sizeof (RBLoadFailureDialog),
-			0,
-			(GInstanceInitFunc) rb_load_failure_dialog_init
-		};
-		
-		rb_load_failure_dialog_type = g_type_register_static (GTK_TYPE_DIALOG,
-								      "RBLoadFailureDialog",
-								      &our_info, 0);
-	}
-
-	return rb_load_failure_dialog_type;
-}
 
 static void
 rb_load_failure_dialog_class_init (RBLoadFailureDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize = rb_load_failure_dialog_finalize;
+
+	g_type_class_add_private (klass, sizeof (RBLoadFailureDialogPrivate));
 }
 
 static void
@@ -103,7 +79,7 @@ rb_load_failure_dialog_init (RBLoadFailureDialog *dlg)
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *gcolumn;
 
-	dlg->priv = g_new0 (RBLoadFailureDialogPrivate, 1);
+	dlg->priv = RB_LOAD_FAILURE_DIALOG_GET_PRIVATE (dlg);
 
 	g_signal_connect_object (G_OBJECT (dlg),
 				 "response",
@@ -176,9 +152,7 @@ rb_load_failure_dialog_finalize (GObject *object)
 
 	g_return_if_fail (dlg->priv != NULL);
 
-	g_free (dlg->priv);
-
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (rb_load_failure_dialog_parent_class)->finalize (object);
 }
 
 GtkWidget *
